@@ -20,7 +20,7 @@
  *  Author:
  *          Tom Zoerner
  *
- *  $Id: epgacqsrv.c,v 1.12 2003/10/05 19:19:37 tom Exp $
+ *  $Id: epgacqsrv.c,v 1.15 2004/06/19 19:52:26 tom Exp $
  */
 
 #define DEBUG_SWITCH DEBUG_SWITCH_EPGCTL
@@ -31,6 +31,10 @@
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
+#include <unistd.h>
+#ifndef WIN32
+#include <sys/types.h>
+#endif
 
 #include "epgctl/mytypes.h"
 #include "epgctl/debug.h"
@@ -218,7 +222,9 @@ static void EpgAcqServer_BuildStatsMsg( EPGDBSRV_STATE * req, bool aiFollows )
             req->msgBuf.stats_ind.u.update.histIdx = pAcqStats->histIdx;
             req->msgBuf.stats_ind.u.update.vpsPdc  = pAcqStats->vpsPdc;
             req->msgBuf.stats_ind.u.update.ttx     = pAcqStats->ttx;
+            req->msgBuf.stats_ind.u.update.stream  = pAcqStats->stream;
             req->msgBuf.stats_ind.u.update.nowNextMaxAcqRepCount = pAcqStats->nowNextMaxAcqRepCount;
+            req->msgBuf.stats_ind.u.update.lastStatsUpdate = pAcqStats->lastStatsUpdate;
          }
       }
 
@@ -519,6 +525,11 @@ static bool EpgAcqServer_TakeMessage( EPGDBSRV_STATE *req, EPGDBSRV_MSG_BODY * p
             req->msgBuf.con_cnf.blockCompatVersion    = DUMP_COMPAT;
             req->msgBuf.con_cnf.protocolCompatVersion = PROTOCOL_COMPAT;
             req->msgBuf.con_cnf.swVersion             = EPG_VERSION_NO;
+#ifndef WIN32
+            req->msgBuf.con_cnf.daemon_pid            = getpid();
+#else
+            req->msgBuf.con_cnf.daemon_pid            = 0;
+#endif
             EpgNetIo_WriteMsg(&req->io, MSG_TYPE_CONNECT_CNF, sizeof(req->msgBuf.con_cnf), &req->msgBuf.con_cnf, FALSE);
 
             req->state = SRV_STATE_WAIT_FWD_REQ;
