@@ -22,7 +22,7 @@
  *
  *  Author: Tom Zoerner
  *
- *  $Id: epgdbmerge.c,v 1.24 2002/01/31 20:28:19 tom Exp tom $
+ *  $Id: epgdbmerge.c,v 1.28 2003/10/05 19:13:36 tom Exp tom $
  */
 
 #define DEBUG_SWITCH DEBUG_SWITCH_EPGDB
@@ -39,7 +39,6 @@
 #include "epgdb/epgdbmgmt.h"
 #include "epgdb/epgtscqueue.h"
 #include "epgdb/epgstream.h"
-#include "epgui/pilistbox.h"
 #include "epgdb/epgdbmerge.h"
 
 
@@ -461,12 +460,13 @@ void EpgDbMergeAllPiBlocks( PDBC dbc )
    EPGDB_BLOCK *pBlock;
    EPGDB_BLOCK *pPrevNetwopBlock[MAX_NETWOP_COUNT];
    time_t firstStartTime, firstStopTime, minStartTime;
-   time_t now;
+   time_t expireTime;
    uchar  netwop, minMappedNetwop, firstNetwop;
    uint dbCount, dbIdx;
 
    dbmc = dbc->pMergeContext;
    dbCount = dbmc->dbCount;
+   expireTime = time(NULL) - dbc->expireDelayPi;
    minStartTime = 0;
    minMappedNetwop = 0;
    memset(pPrevNetwopBlock, 0, sizeof(pPrevNetwopBlock));
@@ -474,7 +474,6 @@ void EpgDbMergeAllPiBlocks( PDBC dbc )
    {
       pNextBlock[dbIdx] = dbmc->pDbContext[dbIdx]->pFirstPi;
    }
-   now = time(NULL);
 
    // loop until all PI blocks are processed
    while (1)
@@ -489,7 +488,7 @@ void EpgDbMergeAllPiBlocks( PDBC dbc )
          pBlock = pNextBlock[dbIdx];
          while ( (pBlock != NULL) &&
                  ( (pBlock->blk.pi.start_time < minStartTime) ||
-                   (pBlock->blk.pi.stop_time < now) ) )
+                   (pBlock->blk.pi.stop_time < expireTime) ) )
          {
             pBlock = pBlock->pNextBlock;
          }
@@ -671,12 +670,13 @@ void EpgDbMergeInsertPi( EPGDB_MERGE_CONTEXT * dbmc, EPGDB_BLOCK * pNewBlock )
 {
    EPGDB_BLOCK *pFoundBlocks[MAX_MERGED_DB_COUNT];
    EPGDB_BLOCK *pWalk, *pPrev, *pNext;
-   time_t now = time(NULL);
 
    // find equivalent blocks in all other dbs and check for conflicts with higher-priorized PI
-   if ( (pNewBlock->blk.pi.stop_time >= now) &&
-        EpgDbMergeGetPiEquivs(dbmc, pNewBlock, pFoundBlocks) )
+   if ( EpgDbMergeGetPiEquivs(dbmc, pNewBlock, pFoundBlocks) )
    {
+      if (pUiDbContext->pPiAcqCb != NULL)
+         pUiDbContext->pPiAcqCb(pUiDbContext, EPGDB_PI_PROC_START, NULL, NULL);
+
       // merge the found blocks
       pNewBlock = EpgDbMergePiBlocks(pUiDbContext, pFoundBlocks);
 
@@ -730,7 +730,8 @@ void EpgDbMergeInsertPi( EPGDB_MERGE_CONTEXT * dbmc, EPGDB_BLOCK * pNewBlock )
 
       assert(EpgDbCheckChains(pUiDbContext));
       // if blocks were removed, re-evaluate scrollbar position
-      PiListBox_DbRecount(pUiDbContext);
+      if (pUiDbContext->pPiAcqCb != NULL)
+         pUiDbContext->pPiAcqCb(pUiDbContext, EPGDB_PI_PROC_DONE, NULL, NULL);
 
       // append the block's covered time range to the PI timescale queue
       if (dbmc->tscEnable)
@@ -826,6 +827,7 @@ void EpgDbMergeAiBlocks( PDBC dbc, uint cfNetwopCount, uint * pNetwopList )
    uint netwop, dbIdx, idx;
    bool  found;
    uchar netOrigIdx[MAX_NETWOP_COUNT];
+   uchar dayCount[MAX_NETWOP_COUNT];
    uchar * pServiceName;          // temporarily holds merged service name
    uint nameLen;                  // sum of netwop name lengths
    uint blockLen;                 // length of composed AI block
@@ -883,6 +885,7 @@ void EpgDbMergeAiBlocks( PDBC dbc, uint cfNetwopCount, uint * pNetwopList )
       if ((netwop >= netwopCount) && (cni != 0) && (cni != 0x00ff) &&
           (netwopCount < MAX_NETWOP_COUNT))
       {
+         dayCount[netwopCount] = 0;
          for (dbIdx=0; dbIdx < dbCount; dbIdx++)
          {
             pAi = (AI_BLOCK *) &dbmc->pDbContext[dbIdx]->pAiBlock->blk.ai;
@@ -901,6 +904,8 @@ void EpgDbMergeAiBlocks( PDBC dbc, uint cfNetwopCount, uint * pNetwopList )
                   pNetwopList[netwopCount] = cni;
                   found = TRUE;
                }
+               if (pNetwops->dayCount > dayCount[netwopCount])
+                  dayCount[netwopCount] = pNetwops->dayCount;
                dbmc->netwopMap[dbIdx][netwop] = netwopCount;
                dbmc->revNetwopMap[dbIdx][netwopCount] = netwop;
             }
@@ -954,6 +959,7 @@ void EpgDbMergeAiBlocks( PDBC dbc, uint cfNetwopCount, uint * pNetwopList )
 
             pTargetNetwops[netwop].cni      = pNetwops->cni;
             pTargetNetwops[netwop].alphabet = pNetwops->alphabet;
+            pTargetNetwops[netwop].dayCount = dayCount[netwop];
             pTargetNetwops[netwop].off_name = blockLen;
             strcpy((char *) AI_GET_STR_BY_OFF(pTargetAi, blockLen), AI_GET_STR_BY_OFF(pAi, pNetwops->off_name));
             blockLen += strlen(AI_GET_STR_BY_OFF(pAi, pNetwops->off_name)) + 1;
