@@ -16,7 +16,7 @@
  *
  *  Author: Tom Zoerner
  *
- *  $Id: piremind.h,v 1.2 2003/08/16 17:12:44 tom Exp tom $
+ *  $Id: piremind.h,v 1.3 2004/06/19 18:58:51 tom Exp tom $
  */
 
 #ifndef __PIREMIND_H
@@ -28,7 +28,4 @@ void PiRemind_Create( void );
 void PiRemind_Destroy( void );
 void PiRemind_CheckDb( void );
 
-uint PiReminder_CtxMenuAdd( Tcl_Interp *interp, const PI_BLOCK * pPiBlock,
-                            const uchar * pMenu, bool addSeparator );
-
 #endif  // __PIREMIND_H
