@@ -1,5 +1,5 @@
 /*
- *  Nextview decoder: xawtv remote control module
+ *  Nextview EPG: xawtv remote control module
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License Version 2 as
@@ -16,22 +16,20 @@
  *
  *  Author: Tom Zoerner
  *
- *  $Id: xawtv.h,v 1.2 2001/05/13 16:28:29 tom Exp tom $
+ *  $Id: xawtv.h,v 1.7 2003/07/06 12:15:08 tom Exp tom $
  */
 
 #ifndef __XAWTV_H
 #define __XAWTV_H
 
 
-// Interface to GUI
-int  Xawtv_GetFreqTab( ulong ** pFreqTab, uint * pCount );
-
-// Interface to main event handler
-void Xawtv_PollVpsPil( ClientData clientData );
-
-// Initialisation
-void Xawtv_Init( void );
+// ----------------------------------------------------------------------------
+// Initialization
+//
+void Xawtv_Init( char * pTvX11Display );
 void Xawtv_Destroy( void );
 
+void Xawtv_SendCmdArgv(Tcl_Interp *interp, const char * pCmdStr, uint cmdLen );
+
 
 #endif  // __XAWTV_H
